
package org.mule.modules.workday.payroll.config.spring;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.modules.workday.payroll.config.PutPayrollFederalW4TaxElectionMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.support.ManagedMap;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

public class PutPayrollFederalW4TaxElectionDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public PutPayrollFederalW4TaxElectionDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(PutPayrollFederalW4TaxElectionMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        if ((element.getAttribute("lockInLetter")!= null)&&(!StringUtils.isBlank(element.getAttribute("lockInLetter")))) {
            builder.addPropertyValue("lockInLetter", element.getAttribute("lockInLetter"));
        }
        if ((element.getAttribute("noWageNoTax")!= null)&&(!StringUtils.isBlank(element.getAttribute("noWageNoTax")))) {
            builder.addPropertyValue("noWageNoTax", element.getAttribute("noWageNoTax"));
        }
        if ((element.getAttribute("nonresidentAlien")!= null)&&(!StringUtils.isBlank(element.getAttribute("nonresidentAlien")))) {
            builder.addPropertyValue("nonresidentAlien", element.getAttribute("nonresidentAlien"));
        }
        if ((element.getAttribute("exempt")!= null)&&(!StringUtils.isBlank(element.getAttribute("exempt")))) {
            builder.addPropertyValue("exempt", element.getAttribute("exempt"));
        }
        if ((element.getAttribute("additionalAmount")!= null)&&(!StringUtils.isBlank(element.getAttribute("additionalAmount")))) {
            builder.addPropertyValue("additionalAmount", element.getAttribute("additionalAmount"));
        }
        if ((element.getAttribute("numberOfAllowances")!= null)&&(!StringUtils.isBlank(element.getAttribute("numberOfAllowances")))) {
            builder.addPropertyValue("numberOfAllowances", element.getAttribute("numberOfAllowances"));
        }
        if ((element.getAttribute("effectiveAsOf")!= null)&&(!StringUtils.isBlank(element.getAttribute("effectiveAsOf")))) {
            builder.addPropertyValue("effectiveAsOf", element.getAttribute("effectiveAsOf"));
        }
        Element workerReferenceListElement = null;
        workerReferenceListElement = DomUtils.getChildElementByTagName(element, "worker-reference");
        List<Element> workerReferenceListChilds = null;
        if (workerReferenceListElement!= null) {
            String workerReferenceRef = workerReferenceListElement.getAttribute("ref");
            if ((workerReferenceRef!= null)&&(!StringUtils.isBlank(workerReferenceRef))) {
                if ((!workerReferenceRef.startsWith(patternInfo.getPrefix()))&&(!workerReferenceRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("workerReference", new RuntimeBeanReference(workerReferenceRef));
                } else {
                    builder.addPropertyValue("workerReference", workerReferenceRef);
                }
            } else {
                ManagedMap workerReference = new ManagedMap();
                workerReferenceListChilds = DomUtils.getChildElementsByTagName(workerReferenceListElement, "worker-reference");
                if (workerReferenceListChilds!= null) {
                    if (workerReferenceListChilds.size() == 0) {
                        workerReferenceListChilds = DomUtils.getChildElements(workerReferenceListElement);
                    }
                    for (Element workerReferenceChild: workerReferenceListChilds) {
                        String workerReferenceValueRef = workerReferenceChild.getAttribute("value-ref");
                        String workerReferenceKeyRef = workerReferenceChild.getAttribute("key-ref");
                        Object valueObject = null;
                        Object keyObject = null;
                        if ((workerReferenceValueRef!= null)&&(!StringUtils.isBlank(workerReferenceValueRef))) {
                            valueObject = new RuntimeBeanReference(workerReferenceValueRef);
                        } else {
                            valueObject = workerReferenceChild.getTextContent();
                        }
                        if ((workerReferenceKeyRef!= null)&&(!StringUtils.isBlank(workerReferenceKeyRef))) {
                            keyObject = new RuntimeBeanReference(workerReferenceKeyRef);
                        } else {
                            keyObject = workerReferenceChild.getAttribute("key");
                        }
                        if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                            keyObject = workerReferenceChild.getTagName();
                        }
                        workerReference.put(keyObject, valueObject);
                    }
                }
                builder.addPropertyValue("workerReference", workerReference);
            }
        }
        Element companyReferenceListElement = null;
        companyReferenceListElement = DomUtils.getChildElementByTagName(element, "company-reference");
        List<Element> companyReferenceListChilds = null;
        if (companyReferenceListElement!= null) {
            String companyReferenceRef = companyReferenceListElement.getAttribute("ref");
            if ((companyReferenceRef!= null)&&(!StringUtils.isBlank(companyReferenceRef))) {
                if ((!companyReferenceRef.startsWith(patternInfo.getPrefix()))&&(!companyReferenceRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("companyReference", new RuntimeBeanReference(companyReferenceRef));
                } else {
                    builder.addPropertyValue("companyReference", companyReferenceRef);
                }
            } else {
                ManagedMap companyReference = new ManagedMap();
                companyReferenceListChilds = DomUtils.getChildElementsByTagName(companyReferenceListElement, "company-reference");
                if (companyReferenceListChilds!= null) {
                    if (companyReferenceListChilds.size() == 0) {
                        companyReferenceListChilds = DomUtils.getChildElements(companyReferenceListElement);
                    }
                    for (Element companyReferenceChild: companyReferenceListChilds) {
                        String companyReferenceValueRef = companyReferenceChild.getAttribute("value-ref");
                        String companyReferenceKeyRef = companyReferenceChild.getAttribute("key-ref");
                        Object valueObject = null;
                        Object keyObject = null;
                        if ((companyReferenceValueRef!= null)&&(!StringUtils.isBlank(companyReferenceValueRef))) {
                            valueObject = new RuntimeBeanReference(companyReferenceValueRef);
                        } else {
                            valueObject = companyReferenceChild.getTextContent();
                        }
                        if ((companyReferenceKeyRef!= null)&&(!StringUtils.isBlank(companyReferenceKeyRef))) {
                            keyObject = new RuntimeBeanReference(companyReferenceKeyRef);
                        } else {
                            keyObject = companyReferenceChild.getAttribute("key");
                        }
                        if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                            keyObject = companyReferenceChild.getTagName();
                        }
                        companyReference.put(keyObject, valueObject);
                    }
                }
                builder.addPropertyValue("companyReference", companyReference);
            }
        }
        Element maritalStatusReferenceListElement = null;
        maritalStatusReferenceListElement = DomUtils.getChildElementByTagName(element, "marital-status-reference");
        List<Element> maritalStatusReferenceListChilds = null;
        if (maritalStatusReferenceListElement!= null) {
            String maritalStatusReferenceRef = maritalStatusReferenceListElement.getAttribute("ref");
            if ((maritalStatusReferenceRef!= null)&&(!StringUtils.isBlank(maritalStatusReferenceRef))) {
                if ((!maritalStatusReferenceRef.startsWith(patternInfo.getPrefix()))&&(!maritalStatusReferenceRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("maritalStatusReference", new RuntimeBeanReference(maritalStatusReferenceRef));
                } else {
                    builder.addPropertyValue("maritalStatusReference", maritalStatusReferenceRef);
                }
            } else {
                ManagedMap maritalStatusReference = new ManagedMap();
                maritalStatusReferenceListChilds = DomUtils.getChildElementsByTagName(maritalStatusReferenceListElement, "marital-status-reference");
                if (maritalStatusReferenceListChilds!= null) {
                    if (maritalStatusReferenceListChilds.size() == 0) {
                        maritalStatusReferenceListChilds = DomUtils.getChildElements(maritalStatusReferenceListElement);
                    }
                    for (Element maritalStatusReferenceChild: maritalStatusReferenceListChilds) {
                        String maritalStatusReferenceValueRef = maritalStatusReferenceChild.getAttribute("value-ref");
                        String maritalStatusReferenceKeyRef = maritalStatusReferenceChild.getAttribute("key-ref");
                        Object valueObject = null;
                        Object keyObject = null;
                        if ((maritalStatusReferenceValueRef!= null)&&(!StringUtils.isBlank(maritalStatusReferenceValueRef))) {
                            valueObject = new RuntimeBeanReference(maritalStatusReferenceValueRef);
                        } else {
                            valueObject = maritalStatusReferenceChild.getTextContent();
                        }
                        if ((maritalStatusReferenceKeyRef!= null)&&(!StringUtils.isBlank(maritalStatusReferenceKeyRef))) {
                            keyObject = new RuntimeBeanReference(maritalStatusReferenceKeyRef);
                        } else {
                            keyObject = maritalStatusReferenceChild.getAttribute("key");
                        }
                        if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                            keyObject = maritalStatusReferenceChild.getTagName();
                        }
                        maritalStatusReference.put(keyObject, valueObject);
                    }
                }
                builder.addPropertyValue("maritalStatusReference", maritalStatusReference);
            }
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
            if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                propertyValues.addPropertyValue("messageProcessors", new ManagedList());
            }
            List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
            listMessageProcessors.add(definition);
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
